@@ -11,15 +11,13 @@
 
 namespace Twig\Extra\Markdown;
 
-use Parsedown;
-
 class ErusevMarkdown implements MarkdownInterface
 {
     private $converter;
 
-    public function __construct(Parsedown $converter = null)
+    public function __construct(?\Parsedown $converter = null)
     {
-        $this->converter = $converter ?: new Parsedown();
+        $this->converter = $converter ?: new \Parsedown();
     }
 
     public function convert(string $body): string
