@@ -13,14 +13,16 @@ namespace Twig\Tests\Extension;
 
 use PHPUnit\Framework\TestCase;
 use Twig\Environment;
+use Twig\Extension\CoreExtension;
 use Twig\Extension\StringLoaderExtension;
+use Twig\Loader\ArrayLoader;
 
 class StringLoaderExtensionTest extends TestCase
 {
     public function testIncludeWithTemplateStringAndNoSandbox()
     {
-        $twig = new Environment($this->createMock('\Twig\Loader\LoaderInterface'));
+        $twig = new Environment(new ArrayLoader());
         $twig->addExtension(new StringLoaderExtension());
-        $this->assertSame('something', twig_include($twig, [], twig_template_from_string($twig, 'something')));
+        $this->assertSame('something', CoreExtension::include($twig, [], StringLoaderExtension::templateFromString($twig, 'something')));
     }
 }
